/**
 ** playExecTaskCodels.c
 **
 ** Codels called by execution task playExecTask
 **
 ** Author: 
 ** Date: 
 **
 **/

#ifdef VXWORKS
# include <vxWorks.h>
#else
# include <portLib.h>
#endif
#include <math.h>
#include "server/playHeader.h"

#include <libplayerc/playerc.h>

/* ------------------------------------------------------------
 * Local variables
 */
static  playerc_client_t *playerClient;
static  playerc_position2d_t *playerPosition2d;
static  playerc_sonar_t *playerSonars;
static  playerc_bumper_t *playerBumpers;

static PLAY_BOOL stopMotion;

/* ------------------------------------------------------------
 * Local functions
 */
static PLAY_BOOL playDisconnect();

/*------------------------------------------------------------------------
 *
 * playExecTaskInit  --  Initialization codel (fIDS, ...)
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
playExecTaskInit(int *report)
{
  sprintf(SDI_F->connect.host, "localhost");
  SDI_F->connect.port = 6665;
  return OK;
}

/*------------------------------------------------------------------------
 *
 * playExecTaskPerm  --  Perm codel
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
playExecTaskPerm(int *report)
{
  if (!SDI_F->status.connectedFlag)
    return OK;
  
  // Read data from the server and display current robot position
  playerc_client_read(playerClient); 
  SDI_F->pos.x = playerPosition2d->px;
  SDI_F->pos.y = playerPosition2d->py;
  SDI_F->pos.theta = playerPosition2d->pa; 
  SDI_F->pos.v = playerPosition2d->vx;
  SDI_F->pos.w = playerPosition2d->va;

  // Robot pos in POM format
  SDI_F->pomMEPos.main.euler.x = playerPosition2d->px;
  SDI_F->pomMEPos.main.euler.y = playerPosition2d->py;
  SDI_F->pomMEPos.main.euler.yaw = playerPosition2d->pa;
  
  // SICK pos on robot : only Z change
#define P3DX_HEIGH 0.215
#define SICK_HEIGH 0.063
  SDI_F->pomSEPos.sensorToMain.euler.z = P3DX_HEIGH + SICK_HEIGH;

  SDI_F->pomSEPos.mainToBase.euler.x = playerPosition2d->px;
  SDI_F->pomSEPos.mainToBase.euler.y = playerPosition2d->py;
  SDI_F->pomSEPos.mainToBase.euler.yaw = playerPosition2d->pa;
  
  SDI_F->pomSEPos.mainToOrigin.euler.x = playerPosition2d->px;
  SDI_F->pomSEPos.mainToOrigin.euler.y = playerPosition2d->py;
  SDI_F->pomSEPos.mainToOrigin.euler.yaw = playerPosition2d->pa;

  return OK;
}


/*------------------------------------------------------------------------
 * Connect
 *
 * Description: 
 *
 * Reports:      OK
 *              S_play_CANNOT_CONNECT
 */

/* playConnectExec  -  codel EXEC of Connect
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
playConnectExec(PLAY_CONNECT *connect, int *report)
{
  int playercEdo;

  // Want to disconnect ?
  if (!connect->connect) {
    playDisconnect();
    SDI_F->status.connectedFlag = PLAY_FALSE;
    return ETHER;
  }

  // Create a client object and connect to the server; the server must
  // be running on "localhost" at port 6665
  playerClient = playerc_client_create(NULL, connect->host, connect->port);
  if (playerc_client_connect(playerClient) != 0)
  {
    fprintf(stderr, "error: %s\n", playerc_error_str());
    *report = S_play_CANNOT_CONNECT;
    return ETHER;
  }

  // Create a position2d proxy (device id "position2d:0") and susbscribe
  // in read/write mode
  playerPosition2d = playerc_position2d_create(playerClient, 0);
  if (playerc_position2d_subscribe(playerPosition2d, PLAYERC_OPEN_MODE) != 0)
  {
    fprintf(stderr, "error: %s\n", playerc_error_str());
    *report = S_play_CANNOT_CONNECT;
    return ETHER;
  }

#if 1
  // One have to get data
  if  (playerc_client_datamode(playerClient,PLAYERC_DATAMODE_PULL)!=0) {
    fprintf(stderr,"error: %s\n",playerc_error_str());
    return ETHER;
  }

  printf ("datamode OK\n");


  // To get last data
  if (playerc_client_set_replace_rule(playerClient,-1,-1,PLAYER_MSGTYPE_DATA,-1,1) !=0) {
    fprintf(stderr,"error: %s\n",playerc_error_str());
    return ETHER; 
  }
#endif

  printf ("lastdata OK\n");

  // create sonar structure
  playerSonars = playerc_sonar_create(playerClient, 0);

  // get sonars configuration
  playerc_sonar_get_geom(playerSonars);

  // bumpers
  playerBumpers =  playerc_bumper_create(playerClient, 0);
  
  playercEdo = playerc_bumper_subscribe(playerBumpers,PLAYERC_OPEN_MODE );
  

  SDI_F->status.connectedFlag = 1;
  return ETHER;
}

/*------------------------------------------------------------------------
 *
 * playExecTaskEnd  --  Endialization codel (fIDS, ...)
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
playExecTaskEnd()
{
  playDisconnect();
  return OK;
}


/*------------------------------------------------------------------------
 * Disconnect
 *
 * Description: 
 *
 * Reports:      PLAY_TRUE/PLAY_FALSE
 */

static PLAY_BOOL playDisconnect()
{
  if (!SDI_F->status.connectedFlag)
    return PLAY_TRUE;

  // Shutdown and tidy up
  playerc_position2d_unsubscribe(playerPosition2d);
  playerc_position2d_destroy(playerPosition2d);
  playerc_client_disconnect(playerClient);
  playerc_client_destroy(playerClient);
  SDI_F->status.connectedFlag = 0;
  return PLAY_TRUE;
}


/*------------------------------------------------------------------------
 * SpeedRef
 *
 * Description: 
 *
 * Reports:      OK
 *              S_play_SPEED_REF_FAIL
 */

/* playSpeedRefStart  -  codel START of SpeedRef
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
playSpeedRefStart(PLAY_SPEED *speed, int *report)
{
  PLAY_BOOL flag=PLAY_TRUE;

  stopMotion = 0;

  // motor on if not yet
  playMotorsExec(&flag, report);

  // Start the robot moving
  playerc_position2d_set_cmd_vel(playerPosition2d, speed->v, 0,speed->w, 1);

  return EXEC;
}


/* playSpeedRefExec  -  codel EXEC of SpeedRef
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
playSpeedRefExec(PLAY_SPEED *speed, int *report)
{
  // make the robot moving
  if (stopMotion)
    return END;

  playerc_position2d_set_cmd_vel(playerPosition2d, speed->v, 0,speed->w, 1);

  return EXEC;
}

/* playSpeedRefInter  -  codel INTER of SpeedRef
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
playSpeedRefInter(PLAY_SPEED *speed, int *report)
{
  playerc_position2d_set_cmd_vel(playerPosition2d, 0, 0, 0, 1);

  return ETHER;
}

/*------------------------------------------------------------------------
 * Motors
 *
 * Description: 
 *
 * Reports:      OK
 *              S_play_FAILED
 */

/* playMotorsExec  -  codel EXEC of Motors
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
playMotorsExec(PLAY_BOOL *flag, int *report)
{
  playerc_position2d_enable(playerPosition2d, 1);
  return ETHER;

  // Already on
  if (*flag && SDI_F->status.motorsFlag)
    return ETHER;

  // Enable the robots motorss
  playerc_position2d_enable(playerPosition2d, *flag);
  SDI_F->status.motorsFlag = *flag;
  return ETHER;
}


/*------------------------------------------------------------------------
 * Stop
 *
 * Description: 
 *
 * Reports:      OK
 */

/* playStopExec  -  codel EXEC of Stop
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
playStopExec(int *report)
{
  // Start the robot moving
  playerc_position2d_set_cmd_vel(playerPosition2d, 0, 0, 0, 1);

  return ETHER;
}


/*------------------------------------------------------------------------
 * Sonars
 *
 * Description: 
 *
 * Reports:      OK
 *              S_play_CANNOT_READ_SONARS
 */

/* playSonarsStart  -  codel START of Sonars
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
playSonarsStart(PLAY_BOOL *sonars, int *report)
{
  // switch off: just leaves the Inter codel to do that
  if (!(*sonars))
    return ETHER;

  // switch on
  if (playerc_sonar_subscribe(playerSonars, PLAYERC_OPEN_MODE)) {
    *report = S_play_CANNOT_READ_SONARS;
    return ETHER;
  }

  return EXEC;
}

/* playSonarsExec  -  codel EXEC of Sonars
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
playSonarsExec(PLAY_BOOL *sonars, int *report)
{
  int i;
  player_pose_t *sc;
  PLAY_POINT *echoR = SDI_F->sonars.echosFrameRobot;
  PLAY_POINT *echoG = SDI_F->sonars.echosFrameGlobal;
  double xR = SDI_F->pos.x;
  double yR = SDI_F->pos.y;
  double tR = SDI_F->pos.theta;

  // fill in header
  SDI_F->sonars.header.nbSensors = playerSonars->scan_count;

  SDI_F->sonars.header.robotPosOdo.x = xR;
  SDI_F->sonars.header.robotPosOdo.y = yR;
  SDI_F->sonars.header.robotPosOdo.theta = tR;

  SDI_F->sonars.header.robotPosGlob.x = xR;
  SDI_F->sonars.header.robotPosGlob.y = yR;
  SDI_F->sonars.header.robotPosGlob.theta = tR;

  // convert echos to robot frame and fill in echos in poster
  // (Number 0 is on front-left, then clockwise)
  for (i=0; i<playerSonars->scan_count; i++) {

    sc = &(playerSonars->poses[i]);

    // robot frame
    echoR[i].x = sc->px + cos(sc->pa)*playerSonars->scan[i];
    echoR[i].y = sc->py + sin(sc->pa)*playerSonars->scan[i];
#if VERBOSE==2
    printf ("%2d: p %4.1f %4.1f %3.0f   r %5.1f   e %4.1f %4.1f\n",
	    i, sc->px, sc->py, sc->pa*180/3.1415925,
	    playerSonars->scan[i], echoR[i].x, echoR[i].y);
#endif

    // global frame
    echoG[i].x = xR + echoR[i].x * cos(tR) - echoR[i].y * sin(tR);
    echoG[i].y = yR + echoR[i].x * sin(tR) + echoR[i].y * cos(tR);
  }

  playSonarsCartPosterWrite(PLAY_SONARSCART_POSTER_ID, (PLAY_SONARSCART_POSTER_STR *)&SDI_F->sonars);

  return EXEC;
}

/* playSonarsEnd  -  codel INTER of Sonars
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
playSonarsInter(PLAY_BOOL *sonars, int *report)
{
  // switch off the sonars
  playerc_sonar_unsubscribe(playerSonars);
  return ETHER;
}

/*------------------------------------------------------------------------
 * TrackSpeed
 *
 * Description: 
 *
 * Reports:      OK
 *              S_play_NONE_POSTER
 */

static POSTER_ID posterID;

/* playTrackSpeedStart  -  codel START of TrackSpeed
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
playTrackSpeedStart(GEN_STRING64 *trackPoster, int *report)
{
  if (posterFind(trackPoster->name,&posterID)==ERROR){
    *report=S_play_NONE_POSTER;
    return ETHER;
  }
  stopMotion = 0;
  return EXEC;
}

/* playTrackSpeedExec  -  codel EXEC of TrackSpeed
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
playTrackSpeedExec(GEN_STRING64 *trackPoster, int *report)
{
  PLAY_SPEED speed;

  if(stopMotion)
    return END;

  if (playPLAY_SPEEDPosterRead(posterID,&speed)==ERROR){
    *report=S_play_CANNOT_READ_POSTER;
    return END;
  }

  playerc_position2d_set_cmd_vel(playerPosition2d, speed.v, 0,speed.w, 1);

  return EXEC;
}

/* playTrackSpeedInter  -  codel INTER of TrackSpeed
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
playTrackSpeedInter(GEN_STRING64 *trackPoster, int *report)
{
  playerc_position2d_set_cmd_vel(playerPosition2d, 0, 0,0, 1);

  return ETHER;
}



/*------------------------------------------------------------------------
 * Bumpers
 *
 * Description: 
 *
 * Reports:      OK
 *              S_play_CANNOT_READ_BUMPERS
 */

/* playBumpersExec  -  codel EXEC of Bumpers
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
playBumpersExec(int *bumperN, int *report)
{
  int i;

  // test each bumper
  for(i=0;i < PLAYERC_BUMPER_MAX_SAMPLES; i++){
    // chock on a bumper
    if(playerBumpers->bumpers[i] == 1){
      *bumperN = i;
      stopMotion = 1;
      return ETHER;
    }
  }
  
  return EXEC;
}

/*------------------------------------------------------------------------
 * Laser
 *
 * Description: 
 *
 * Reports:      OK
 *              S_play_CANNOT_SWITCH_ON
 */

/* playLaserStart  -  codel START of Laser
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
playLaserStart(PLAY_BOOL *laserFlag, int *report)
{
  playerc_laser_t *device;
  device = playerc_laser_create(playerClient, index);
  playerc_laser_subscribe(device, PLAYER_OPEN_MODE);
  return ETHER;
}




