#ifndef PLAY_STRUCT_H
#define PLAY_STRUCT_H

typedef enum PLAY_BOOL {
  PLAY_FALSE,
  PLAY_TRUE
} PLAY_BOOL;

/* ------------------------------------------------------------
 * CONNECTION WITH PLAYER SERVER
 */

/* Connection to player server */
typedef struct PLAY_CONNECT {
  PLAY_BOOL connect; /* want to connect or disconnect ? */
  int port;          /* port number (uselly 6665) */
  char host[64];     /* machine where player server runs */
} PLAY_CONNECT;

typedef struct PLAY_STATUS {
  PLAY_BOOL  connectedFlag;  /* connection to player on */
  PLAY_BOOL  motorsFlag;     /* motors on */
  PLAY_BOOL  sonarsFlag;     /* sonars connected */
  PLAY_BOOL  bumpersFlag;    /* bumpers connected */
  PLAY_BOOL  laserFlag;      /* laser connected */
  PLAY_BOOL  unused;         /* for alignment purpose */
} PLAY_STATUS;


/* ------------------------------------------------------------
 * POSITION AND CONFIGURATION
 */
typedef struct PLAY_POINT {
  double x;
  double y;
} PLAY_POINT;

typedef struct PLAY_POS {
  double x;
  double y;
  double theta;
} PLAY_POS;

typedef struct PLAY_CONFIG {
  double x;
  double y;
  double theta;
  double v;
  double w;
} PLAY_CONFIG;

typedef struct PLAY_SPEED {
  double v;
  double w;
} PLAY_SPEED;


/* ------------------------------------------------------------
 * SENSORS
 */
typedef struct PLAY_SENSORS_HEADER {
  int time;
  int nbSensors;
  PLAY_CONFIG robotPosOdo;
  PLAY_CONFIG robotPosGlob;
} PLAY_SENSORS_HEADER;


/* ------------------------------------------------------------
 * SONARS
 * Number 0 is on front-left, then clockwise 
 */

/* XXX should be read from player server
 * Number 0 is on front-left, then clockwise */ 
#define PLAY_NB_MAX_SONARS 16

typedef struct PLAY_SONARS {
  PLAY_SENSORS_HEADER header;
  PLAY_POINT         echosFrameRobot[PLAY_NB_MAX_SONARS];
  PLAY_POINT         echosFrameGlobal[PLAY_NB_MAX_SONARS];
} PLAY_SONARS;


/* ------------------------------------------------------------
 * BUMPERS
 */
#define PLAY_NB_MAX_BUMPERS 10

typedef struct PLAY_BUMPERS {
  PLAY_SENSORS_HEADER header;
  PLAY_BOOL bump[PLAY_NB_MAX_BUMPERS];
} PLAY_BUMPERS;

#endif
